@@ -11,6 +11,8 @@ fn main() {
 fn load_execute() -> Result<(), Box<dyn Error>> {
     let  a: Vec<String> = env::args().collect();
     let mut c = CPU::new();
+    c.debug.switch = true;
+
     // Loads assembled program into memory
     c.bus.load_bin(&a[1], 0x000)?;
 
@@ -20,6 +22,7 @@ fn load_execute() -> Result<(), Box<dyn Error>> {
 
     loop {
         c.execute();
+        println!("{}\n", c.debug.string);
         if c.pc == 0x0000 { break }             //  if CP/M warm boot -> we exit
     }
     Ok(())
